% scripts to generate the data of the graphical example as in the paper: 
% T. L. Griffiths and M. Steyvers. Finding scientific topics. Proceedings of
% the National Academy of Sciences of U.S., 2004.
% 
% Copyright (C) 2009 Xiaodong Yu, xdyu at umiacs umd edu;
% distributable under GPL, see README.txt

%% 16 word 
figure; colormap 'gray'
imagesc(reshape(zeros(16,1), [4 4])); axis equal; axis tight; %axis off;
title('Bag of Words - each sqaure is a word');
grid on;
set(gca, 'xtick', [-0.5:1:4]);
set(gca, 'ytick', [-0.5:1:4]);
print('-dpng', 'pics/words.png');

%% 8 topic
% document topic mixture
% ground truth of topic-specified word distribution,  Phi
% the size of the vocabulary is 16 = 4x4, 
% i.e., each position represent a word

Phi(:,1) = [1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0]/4;
Phi(:,2) = [0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0]/4;
Phi(:,3) = [0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0]/4;
Phi(:,4) = [0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1]/4;
Phi(:,5) = [1 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0]/4;
Phi(:,6) = [0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 0]/4;
Phi(:,7) = [0 0 0 0 0 0 0 0 1 1 1 1 0 0 0 0]/4;
Phi(:,8) = [0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1]/4;

% show ground truth of Phi
figure; colormap 'gray'
for i=1:8
    subplot(2,4,i); imagesc(reshape(Phi(:,i), [4 4])); axis equal; axis tight; % axis off;
    title(['Topic \theta_' num2str(i)])
end;
print('-dpng', 'pics/topic_theta_ground_truth.png');

%% hyperparameter: assume symmetric Dirichlet distribution
alpha = 5;   % document  dir process  ٸ Ϸ vector    ʹ.
beta  = 1;   % topic  dir process ٸ Ϸ vector    ʹ.

M     = 500; % number of document
K     = 8;   % number of topic
Nd    = 100; % number of words in each document
V     = 16;  % size of the vocabulary

%% ground truth of document-specified topic distribution,  Theta
z = zeros(M,Nd);
w = zeros(M,Nd);
word_hist = zeros(4,4,M);
Theta = zeros(M,K);

for d = 1:M % document
    theta = dirrnd(ones(1,K) * alpha); % topic distbituion in a document
    
    for n = 1:Nd % words in each document,   ܾ ȣ  bin ȣ ƴϴ
        
        %% draw topic for each word, Multinomial random numbers
        z(d,n) = find(mnrnd(1,theta)==1); % theta -> z
        
        %% draw word
        m_p    = reshape(Phi(:,z(d,n)),[1 V]); % theta -> z, ش topic ܾ distribution
        w(d,n) = find(mnrnd(1,m_p)==1); % z -> w, % topic dist  ܾ  multinomial probability
        % Phi   ٰ ,  dir mul  ̷κ ܾ 
        % ܾ ȣ 
    end;
    word_hist(:,:,d) = reshape(hist(w(d,:),16), [4 4]); 
    %word_hist BOW     ִ  . ׷ װɷ ű⿡ ´ Phi ϴ  ?
    %Docs_words = (m,:) = hist2elem(Docs(:,:,m));
    Theta(d,:) = theta;
end;

%% show exmple documents
figure; colormap 'gray'
plotw = 4;
ploth = 5;
for i = 1:plotw*ploth
    subplot(plotw, ploth, i); 
    imagesc(word_hist(:,:,i)); axis equal; axis tight; %axis off;
    %colorbar
    title(['' num2str(i) '-th doc']);
end;
print('-dpng', 'pics/example_documents.png');

%%
save('lda_test_data.mat','word_hist','w','z','Theta','Phi','K', 'alpha','beta');

