function [Phi, Theta, logPw_z, Phi2, Theta2]=learn_GibbsLDA(w, K, alpha, beta, max_iter, ...
    burn_in_iter, sampling_lag, plot_flag, plot_phi_sizes)
% function learn_GibbsLDA
% load demo_data;
% alpha = 5;
% beta = 1;
% max_iter = 1000;
% burn_in_iter = 100;
% sampling_lag = 50; %50 ѹ sample ?, burn_in period
% plot_flag = 1;
% plot_phi_sizes = [4 4];

% function [Phi, Theta, z]=learn_GibbsLDA(w, K, alpha, beta)
% To do: perform Gibbs sampling for LDA inference
%
% Parameters:
%% w:
% a M x Nd matrix that stores all the words in the M documents, one document each row
% In this simple demo, we assume the number of words in each document
% is the same, i.e., Nd
%% K: number of topics
%% alpha :
% hyperparameter of Dirichlet prior for Theta, represent the prior count of
% topics in a document
%% beta :
% hyperparameter of Dirichlet prior for Phi, represent the prior
% count of words in a topic
% I assume symmetric Dirichlet distribution, so both alpha and beta are scalars
%% max_iter: maximum iterations
%% burn_in_iter: burn-in iterations
%% sampling_lag: sampling lag between two read-out or figure-showing
%% plot_flag:
% flag to show figures or not; if show figures, they will also
% be saved on local foloders
%% plot_phi_sizes: the size to reshape the Phi in order to show it
%
%% Output:
% Phi: a V x K matrix, each column is a topic-specified word distribution
% Theta: a K x M matrix, each column is a document-specified topic distribution
%
%% dimensionalities
% M : number of document
% K : number of topic
% V : size of vocabulary
% Nd : number of words in each document
%
%% example usage:
% [est_Phi, est_Theta, logPw_z]=learn_GibbsLDA(w, K, alpha, beta, 1000, 200, 100, 1, [4 4]);
%
%% Reference:
% The equations of the conditional posterior is based on the technical note
% by Yi Wang, "Gibbs Sampling and Latent Diriclet Allocation: The Gritty Details ",
% available from http://dbgroup.cs.tsinghua.edu.cn/wangyi/lda/index.html
%
% Copyright (C) 2009 Xiaodong Yu, xdyu at umiacs umd edu;
% distributable under GPL, see README.txt


%% 1. Gibbs sampling   ʱȭѴ. 
fprintf('1. Initialize parameters for Gibbs sampling ');
[D, Nd] = size(w);
V       = length(unique(w));
NWZ     = zeros(V, K) + beta; % Phi
NZM     = zeros(K, D) + alpha; % Theta
NZ      = sum(NWZ); % to accelerate the computation, V vocabulary , ѹ  topic  ܾ 
% topic distribution  ϱ  ε,
z       = zeros(D, Nd);
Phi     = zeros(V, K);
Theta   = zeros(K, D);

for d = 1:D % for each document
    
    for n = 1:Nd % for each word
        
       %% Z is "topic allocation vector z" 
        z(d,n) = find(mnrnd(1,ones(1,K)/K )==1); 
        % Document ִ  ܾ鿡  topic idx Ѵ. 
        
       %% NZM is \Omega, [Document-Topic]
        NZM(z(d,n), d) = NZM(z(d,n), d) + 1;
        % ش Document ش topic  ϳ Ų. 
        
       %% NWZ is \Phi, [Topic-Word]
        NWZ(w(d,n),z(d,n)) = NWZ(w(d,n),z(d,n)) + 1;
        % ش Word Topic  ϳ Ų. 
        
       %% NZ [Topic]
        NZ(z(d,n)) = NZ(z(d,n)) + 1;
        % ش Topic  ϳ Ų. 
        
    end;
end;
fprintf('\n');

%% 2. Phi ʱ value ش. 
if plot_flag == 1
    figure(); clf; colormap 'gray';
    for k = 1:K
        %  ʱ topic  ׸. 
        subplot(2, K/2, k);
        Phi(:,k) = NWZ(:,k)/NZ(k); % Normalize 
        imagesc(reshape(Phi(:,k), plot_phi_sizes)); axis equal; axis tight;
        title(['Topic \phi_' num2str(k)])
    end;
end;


%% 3. Start Gibbs sampling for LDA 
fprintf('2. Start Gibbs sampling');
read_out_Phi        = zeros(V, K); % Word-Topic
read_out_Theta      = zeros(K, D); % Topic-Document
read_out_sampling_num = 0;
logPw_z             = zeros(1, max_iter);

for iter = 1:max_iter
    fprintf('.');
    
    for d = 1:D % for each document
        for n = 1:Nd % for each word
            
           %% 1. Decrease three counts corresponding to current document and word
            % It is for excluding currently sampling latent variable in Gibbs sampling 
            NZM(z(d,n),d)       = NZM(z(d,n),d) - 1;      % NZM is \Omega, [Document-Topic] #words in topic in doc
            NWZ(w(d,n),z(d,n))  = NWZ(w(d,n),z(d,n)) - 1; % NWZ is \Psi, [Topic-Word] #words in topic
            NZ(z(d,n))          = NZ(z(d,n)) -1;          % NZ [Topic]: #words in topic
            
           %% 2. Update the posterior distribution of z, p(z_i), eq (2.40)
            p  = zeros(1, K);
            for k = 1:K
                p(k) = NWZ(w(d,n), k) / NZ(k) * NZM(k,d);
            end;
            p = p/sum(p);
            
           %% 3. Draw topic for this word
            z(d,n) = find(mnrnd(1, p)==1);
            
           %% 4. Increase three counts eq (2.30)
            NZM(z(d,n),d)      = NZM(z(d,n),d) + 1;
            NWZ(w(d,n),z(d,n)) = NWZ(w(d,n),z(d,n)) + 1;
            NZ(z(d,n))         = NZ(z(d,n)) + 1;
            
        end;
    end;
    
    %% Evaluation  log likelihood Ѵ. p(w|z) eq (2.20)
    for zz = 1:K
        logPw_z(iter) = logPw_z(iter) + log_multinomial_beta(NWZ(:,zz)) - log_multinomial_beta(ones(1,V)*beta);
    end;
    
    %% % sampling_lag ѹ sample ؼ  Ѵ. (߿ )
    if mod(iter, sampling_lag) == 0 || iter == 1
        if iter >= burn_in_iter 
            % burn-in Ŀ 
            read_out_sampling_num = read_out_sampling_num + 1;
            for k=1:K
                read_out_Phi(:,k) = read_out_Phi(:,k) + NWZ(:,k)/NZ(k);
            end;
            for mm = 1:D
                read_out_Theta(:,mm) = read_out_Theta(:,mm) + NZM(:,mm)/sum(NZM(:,mm));
            end;
        end;
        
       %% show the current Phi
        if plot_flag == 1
            figure(5); clf;
            colormap 'gray'
            for k = 1:K
                subplot(2, K/2, k);
                Phi(:,k) = NWZ(:,k)/NZ(k);
                imagesc(reshape(Phi(:,k), plot_phi_sizes)); axis equal; axis tight; axis off;
                title(['Topic dist \theta_' num2str(k)])
            end;
            fig_fname = ['pics/Phi_iter_' num2str(iter) '.jpg'];
            print('-djpeg', fig_fname);
        end;
        fprintf('\n'); 
    end;
end
fprintf('\n');

%% 4.  Phi Theta Ѵ. 
Phi   = read_out_Phi/read_out_sampling_num;    % [Word - Topic]
Theta = read_out_Theta/read_out_sampling_num;  % [Topic - Document]

Phi2    = zeros(V, K);
Theta2  = zeros(K, D);
for k = 1:K
    Phi2(:, k)   = NWZ(:,k)/NZ(k);    % [Word - Topic]
end
for mm = 1:D
    Theta2(:, mm) = NZM(:,mm)/sum(NZM(:,mm));  % [Topic - Document]
end

if plot_flag == 1
    % show the final results
    figure(1); clf;
    colormap 'gray'
    for k=1:K
        subplot(1,K,k);
        imagesc(reshape(Phi(:,k), plot_phi_sizes)); axis equal; axis tight; axis off;
        %title(['\phi_' num2str(k)])
    end;
    fig_fname = 'final_Phi_iter.jpg';
    print('-djpeg', fig_fname);
    
    % show the history of log likelihood, log(w|z), solid red dots
    % corresponding to the iterations when the figures
    figure(2); clf;
    plot(logPw_z,'b-');
    hold on;
    plot(sampling_lag:sampling_lag:max_iter, ...
        logPw_z(sampling_lag:sampling_lag:max_iter),'ro',...
        'MarkerSize', 10, 'MarkerFaceColor', 'r');
    xlabel('iterations'); ylabel('log(P(w|z))');
    print('-djpeg', 'logPw_z.jpg');
    
end;

%%


function b = log_multinomial_beta(alpha) % eq (2.5)
%% compute log of multinomial beta function, a multivariate extension of beta function
L = length(alpha);
b = 0;
for i=1:L
    b = b + (gammaln(alpha(i)));
end;
b = b-gammaln(sum(alpha)); 

